#!/usr/bin/env ruby

require 'cgi'
require 'net/http'
require 'ostruct'
require 'rubygems'
require 'stringio'
require 'uri'
require 'date'

if ARGV.empty?
  puts "Need a Filename!"
  exit
else
  file = ARGV.first
end

# get movie/tv show details by parsing filename
def get_movie_details(file)
  movie = OpenStruct.new
  movie.file = file
  
  # parse filename
  name = file
  if name =~ /[sS]{0,1}[0-9]{1,2}[eExX][0-9]{1,2}/ # nice episode "S03E12" or "4x02"
    movie.kind = :tvshow
    file_match = name.match(/[sS]{0,1}[0-9]{1,2}[eExX][0-9]{1,2}/)
    movie.show = file_match.pre_match.gsub(".", " ").strip
    movie.episode = file_match.to_s.match(/[0-9]{1,2}$/).to_s.to_i
    movie.season = file_match.to_s.match(/[0-9]{1,2}$/).pre_match.match(/[0-9]{1,2}/).to_s.to_i
    movie.episode_id = movie.season.to_s + (movie.episode < 10 ? "0" + movie.episode.to_s : movie.episode.to_s)
    movie.name = "Episode #{movie.episode.to_s}"
    movie.year = Time.now.year
  elsif name =~ /\D[0-9][0-9][0-9]\D/ # bad espisode "411"
    movie.kind = :tvshow
    file_match = name.match(/\D[0-9][0-9][0-9]\D/)
    movie.show = file_match.pre_match.gsub(".", " ").strip
    movie.episode = file_match.to_s.match(/[0-9][0-9]\D$/).to_s.to_i
    movie.season = file_match.to_s.match(/[0-9][0-9]\D$/).pre_match.match(/[0-9]{1,2}/).to_s.to_i
    movie.episode_id = movie.season.to_s + (movie.episode < 10 ? "0" + movie.episode.to_s : movie.episode.to_s)
    movie.name = "Episode #{movie.episode.to_s}"
    movie.year = Time.now.year
  elsif name =~ /[0-9][0-9][0-9][0-9]/ # year number. should be a movie
    movie.kind = :movie
    movie.name = name.match(/[\[\(]{0,1}[0-9][0-9][0-9][0-9]/).pre_match.gsub(".", " ").strip
    movie.year = name.match(/[0-9][0-9][0-9][0-9]/).to_s.to_i
  else
    movie.kind = :movie
    movie.name = name
    movie.year = Time.now.year
  end
    
  if movie.kind == :tvshow
  	  movie = get_details_from_tvrage(movie)
  end
  
  #put plist header
  puts "<?xml version='1.0' encoding='UTF-8'?>
	<!DOCTYPE plist SYSTEM 'file://localhost/System/Library/DTDs/PropertyList.dtd'>
	<plist version='0.9'>
	<dict>"
  puts "<key>video kind</key>"
  puts "<string>#{movie.kind}</string>"
  if movie.name != nil
  	puts "<key>name</key>"
  	puts "<string>#{movie.name}</string>"
  end
  if movie.show != nil
  	puts "<key>show</key>"
  	puts "<string>#{movie.show}</string>"
  end
  if movie.season != nil
  	puts "<key>season</key>"
  	puts "<string>#{movie.season}</string>"
  end
  if movie.episode != nil
  	puts "<key>episode</key>"
  	puts "<string>#{movie.episode}</string>"
  end
  if movie.episode_id != nil
  	puts "<key>episode_id</key>"
  	puts "<string>#{movie.episode_id}</string>"
  end
  if movie.year != nil
  	puts "<key>year</key>"
  	puts "<string>#{movie.year}</string>"
  end
  if movie.genre != nil
  	puts "<key>genre</key>"
  	puts "<string>#{movie.genre}</string>"
  end
  if movie.url != nil
  	puts "<key>url</key>"
  	puts "<string>#{movie.url}</string>"
  end
  puts "</dict>
</plist>"
 
  movie
end

# fetch tvshow details form tvrage.com
def get_details_from_tvrage(show)
  # prepare URI
  #url = URI::HTTP.build(:host => 'www.tvrage.com', :path => '/quickinfo.php', :query => ('show=' + CGI.escape(show.show) + '&ep=' + show.season.to_s + 'x' + show.episode.to_s))
   url = URI::HTTP.build(:host => 'services.tvrage.com', :path => '/tools/quickinfo.php', :query => ('show=' + CGI.escape(show.show) + '&ep=' + show.season.to_s + 'x' + show.episode.to_s))
  begin
    response = StringIO.new(Net::HTTP.get(url))
    while !response.eof?
      line = response.gets
      line = line.split(/[@\n\^\|]/)
	#puts line.inspect
      case line.first
      when 'Show Name'
        show.show = line[1]
      when 'Show URL'
        show.url = line[1]
      when 'Genres'
        show.genre = line[1].strip # taking only first into account
      when 'Episode Info'
        show.name = line[2]
        begin
         show.year = Date.parse(line[3]).year
        rescue
        end
      end
    end
  rescue
  end

  show
end

# get details for movie
movie = get_movie_details(file)
